@@ -74,7 +74,7 @@ class RelativePositionalEmbedding(nn.Module):
 # main class
 
 class LocalAttention(nn.Module):
-    def __init__(self, window_size, causal = False, look_backward = 1, look_forward = None, dropout = 0., shared_qk = False, rel_pos_emb_config = None, autopad = False):
+    def __init__(self, window_size, causal = False, look_backward = 1, look_forward = None, dropout = 0., shared_qk = False, rel_pos_emb_config = None, autopad = False, exact_windowsize = False):
         super().__init__()
         look_forward = default(look_forward, 0 if causal else 1)
         assert not (causal and look_forward > 0), 'you cannot look forward if causal'
@@ -83,6 +83,7 @@ class LocalAttention(nn.Module):
         self.causal = causal
         self.look_backward = look_backward
         self.look_forward = look_forward
+        self.exact_windowsize = exact_windowsize
         self.autopad = autopad
 
         self.dropout = nn.Dropout(dropout)
@@ -97,6 +98,8 @@ class LocalAttention(nn.Module):
             self.rel_pos = RelativePositionalEmbedding(dim_head, heads, rel_pos_length)
 
     def forward(self, q, k, v, input_mask = None):
+        shape = q.shape
+
         merge_into_batch = lambda t: t.reshape(-1, *t.shape[-2:])
         q, k, v = map(merge_into_batch, (q, k, v))
 
@@ -105,7 +108,7 @@ class LocalAttention(nn.Module):
             q, k, v = map(lambda t: pad_to_multiple(t, self.window_size, dim = -2), (q, k, v))
 
         window_size, causal, look_backward, look_forward, shared_qk = self.window_size, self.causal, self.look_backward, self.look_forward, self.shared_qk
-        b, t, e, device, dtype, shape = *q.shape, q.device, q.dtype, q.shape
+        b, t, e, device, dtype = *q.shape, q.device, q.dtype
         assert (t % window_size) == 0, f'sequence length {t} must be divisible by window size {window_size} for local attention'
 
         windows = t // window_size
@@ -144,6 +147,11 @@ class LocalAttention(nn.Module):
             dots.masked_fill_(mask, mask_value)
             del mask
 
+        if causal and self.exact_windowsize:
+            mask = (bq_t[:, :, :, None] - bq_k[:, :, None, :]).abs() > (self.window_size * self.look_backward)
+            dots.masked_fill_(mask, mask_value)
+            del mask
+
         mask = bq_k[:, :, None, :] == -1
         dots.masked_fill_(mask, mask_value)
         del mask
@@ -164,9 +172,9 @@ class LocalAttention(nn.Module):
         attn = self.dropout(attn)
 
         out = torch.einsum('bhij,bhje->bhie', attn, bv)
-        out = out.reshape(*shape)
+        out = out.reshape(-1, t, e)
 
         if self.autopad:
-            return out[..., :orig_t, :]
+            out = out[:, :orig_t, :]
 
-        return out
+        return out.reshape(*shape)
